# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

import icemac.ab.importer.interfaces
import unittest
import zope.interface.verify
import icemac.ab.importer.reader.base


class BaseReaderTest(unittest.TestCase):
    "Base class for reader tests."

    def test_interfaces(self):
        zope.interface.verify.verifyObject(
            icemac.ab.importer.interfaces.IImportFileReader,
            icemac.ab.importer.reader.base.BaseReader())

    def test_canRead_exception_in_open(self):
        class Reader(icemac.ab.importer.reader.base.BaseReader):

            @classmethod
            def open(cls, file):
                raise Exception()

        self.assertEqual(False, Reader.canRead(None))

    def test_canRead_exception_in_getFieldNames(self):
        class Reader(icemac.ab.importer.reader.base.BaseReader):

            def getFieldNames(self):
                raise Exception()

        self.assertEqual(False, Reader.canRead(None))

    def test_canRead_getFieldNames_returns_empty_list(self):
        class Reader(icemac.ab.importer.reader.base.BaseReader):

            def getFieldNames(self):
                return []

        self.assertEqual(False, Reader.canRead(None))

    def test_canRead_getFieldNames_returns_nonempty_list(self):
        class Reader(icemac.ab.importer.reader.base.BaseReader):

            def getFieldNames(self):
                return ['sdfg']

        self.assertEqual(True, Reader.canRead(None))



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

import datetime
import icemac.ab.importer.interfaces
import os.path
import sys
import unittest
import zope.interface.verify


class ReaderTest(unittest.TestCase):
    "Test class for readers."

    reader_class = None # reference to the reader's class object
    import_file = None # name of the longer import file
    import_file_short = None # name of the shorter import file

    def getFileHandle(self, file_name=None):
        base_path = sys.modules[self.reader_class.__module__].__file__
        if file_name is None:
            file_name = self.import_file
        return file(os.path.join(
                os.path.dirname(base_path), 'tests', 'data', file_name))

    def getReader(self, import_file=None):
        return self.reader_class.open(self.getFileHandle(import_file))

    def test_interfaces(self):
        zope.interface.verify.verifyObject(
            icemac.ab.importer.interfaces.IImportFileReader,
            self.getReader())

    def test_getFieldNames(self):
        field_names = list(self.getReader().getFieldNames())
        self.assertEqual([u'last name', u'firstname', u'birth_date'],
                         field_names)
        self.assert_(isinstance(field_names[0], unicode))
        self.assert_(isinstance(field_names[1], unicode))
        self.assert_(isinstance(field_names[2], unicode))

    def test_getFieldSamples_firstname(self):
        samples = list(self.getReader().getFieldSamples(u'firstname'))
        self.assertEqual([u'Andreas', u'Hanna', u'Jens'], samples)
        self.assert_(isinstance(samples[0], unicode))
        self.assert_(isinstance(samples[1], unicode))
        self.assert_(isinstance(samples[2], unicode))

    def test_getFieldSamples_lastname(self):
        self.assertEqual([u'Koch', u'Hula', u'Jänsen'],
                         list(self.getReader().getFieldSamples(u'last name')))

    def test_getFieldSamples_birthdate(self):
        samples = list(self.getReader().getFieldSamples(u'birth_date'))
        self.assertEqual([u'1976-01-24', u'2000-01-01', u''], samples)
        self.assert_(isinstance(samples[0], unicode))
        self.assert_(isinstance(samples[1], unicode))
        self.assert_(isinstance(samples[2], unicode))

    def test_getFieldSamples_less_than_3_samples_in_file(self):
        samples = list(self.getReader(self.import_file_short).getFieldSamples(
                u'last name'))
        self.assertEqual([u'Koch'], samples)
        self.assert_(isinstance(samples[0], unicode))

    def test_getFieldSamples_empty_string(self):
        samples = list(self.getReader(self.import_file_short).getFieldSamples(
                u'firstname'))
        self.assertEqual([u''], samples)
        self.assert_(isinstance(samples[0], unicode))

    def test___iter__(self):
        result = [[u'Koch', u'Andreas', datetime.date(1976, 1, 24)],
                  [u'Hula', u'Hanna', datetime.date(2000, 1, 1)],
                  [u'Jänsen', u'Jens', None],
                  [u'Fruma', None, datetime.date(2001, 12, 31)]]
        for index, line in enumerate(self.getReader()):
            self.assertEqual(result[index], line)
            for value in line:
                self.assert_((isinstance(value, unicode) or
                              isinstance(value, datetime.date) or
                              value is None),
                             repr(value))

    def test___iter__short(self):
        result = [[u'Koch', None, datetime.date(1976, 1, 24)]]
        for index, line in enumerate(self.getReader(self.import_file_short)):
            self.assertEqual(result[index], line)
            for value in line:
                self.assert_((isinstance(value, unicode) or
                              isinstance(value, datetime.date) or
                              value is None),
                             repr(value))



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import datetime
import decimal
import gocept.reference.field
import icemac.ab.importer.browser.wizard.base
import icemac.ab.importer.interfaces
import icemac.addressbook.interfaces
import persistent.mapping
import time
import z3c.form.field
import zc.sourcefactory.contextual
import zope.component
import zope.interface
import zope.schema
import zope.security.proxy


NONE_REPLACEMENT = object()
TRUE_VALUES = ['yes', 'true']
FALSE_VALUES = ['no', 'false']
DATETIME_FORMAT = '%Y-%m-%d %H:%M'


def get_reader(session):
    if not hasattr(session, 'cache'):
        # we are in a dict below the session (storing field group
        # data), so we have to use the "uplink"
        session = session['__parent__']
    reader = session.cache.get('reader', None)
    if reader is not None:
        return reader
    reader_name = session['reader']
    reader = zope.component.getAdapter(
        None,
        icemac.ab.importer.interfaces.IImportFileReader,
        name=reader_name)
    file = zope.security.proxy.removeSecurityProxy(session.file.openDetached())
    reader = reader.open(file)
    session.cache['reader'] = reader
    return reader


class ImportFields(zc.sourcefactory.contextual.BasicContextualSourceFactory):
    "Where to put the imported data in the addressbook."

    def getValues(self, context):
        return xrange(len(list(get_reader(context).getFieldNames())))

    def getTitle(self, context, value):
        reader = get_reader(context)
        field_name = list(reader.getFieldNames())[value]
        samples = u', '.join(
            x for x in reader.getFieldSamples(field_name) if x)
        title = field_name
        if samples:
            title = '%s (%s)' % (field_name, samples)
        return title

import_fields = ImportFields()

def split_keywords(keywords):
    return [x.strip() for x in keywords.split(',')]

class IFieldValue(zope.interface.Interface):
    "Value for a specific field."

    def __call__():
        "Return the value."


# field value converters
@zope.component.adapter(None, None)
@zope.interface.implementer(IFieldValue)
def unchanged_field(value, field):
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    return value


@zope.interface.implementer(IFieldValue)
def text_field(value, field):
    "Adapter for `Text`, `TextLine` and `Choice` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    return value.strip()


@zope.component.adapter(None, zope.schema.interfaces.IInt)
@zope.interface.implementer(IFieldValue)
def int_field(value, field):
    "Adapter for `Int` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    try:
        return int(float(value))
    except (TypeError, ValueError):
        return value


@zope.component.adapter(None, zope.schema.interfaces.IDecimal)
@zope.interface.implementer(IFieldValue)
def decimal_field(value, field):
    "Adapter for `Decimal` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    try:
        return decimal.Decimal(value)
    except (TypeError, ValueError, decimal.InvalidOperation):
        return value


@zope.component.adapter(None, zope.schema.interfaces.IBool)
@zope.interface.implementer(IFieldValue)
def bool_field(value, field):
    "Adapter for `Bool` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    if not isinstance(value, unicode):
        return value # produces an error in validation
    value = value.lower()
    if value in TRUE_VALUES:
        return True
    if value in FALSE_VALUES:
        return False
    return value


@zope.component.adapter(None, zope.schema.interfaces.IDatetime)
@zope.interface.implementer(IFieldValue)
def datetime_field(value, field):
    "Adapter for `Datetime` fields."
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    try:
        time_tuple = time.strptime(value.strip(), DATETIME_FORMAT)
    except (TypeError, ValueError):
        return value
    else:
        return datetime.datetime(*time_tuple[:5])


@zope.component.adapter(None, zope.schema.interfaces.IURI)
@zope.interface.implementer(IFieldValue)
def uri_field(value, field):
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    return value.strip().encode('ascii')


@zope.component.adapter(None, zope.schema.interfaces.IChoice)
@zope.interface.implementer(IFieldValue)
def country_field(value, field):
    if value is None:
        # We can't return None, as this means that the adapter can't adapt.
        return NONE_REPLACEMENT
    value = value.strip()
    for candidate in field.source.factory.getValues():
        if candidate.token == value:
            return candidate
        if field.source.factory.getTitle(candidate) == value:
            return candidate
    # value not found in source --> can't adapt
    return None


@zope.component.adapter(None, gocept.reference.field.Set)
@zope.interface.implementer(IFieldValue)
def keywords_field(value, field):
    keywords = set()
    if value is None:
        return keywords
    keyword_util = zope.component.getUtility(
        icemac.addressbook.interfaces.IKeywords)
    for keyword_title in split_keywords(value):
        keywords.add(keyword_util.get_keyword_by_title(keyword_title))
    return keywords


class IErrorMessage(zope.interface.Interface):
    """Render import error message."""

    def __unicode__():
        """Return error text."""


# Error renderers
@zope.interface.implementer(IErrorMessage)
def simple_invalid(field, exc):
    return exc.doc()


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        zope.schema.interfaces.ConstraintNotSatisfied)
@zope.interface.implementer(IErrorMessage)
def choice_constraint_not_satisfield(field, exc):
    value = exc.args[0]
    if zope.schema.interfaces.IVocabulary.providedBy(field.source):
        allowed = [x.value for x in field.source]
    else:
        allowed = [str(x) for x in field.source.factory.getValues()]
    return _(u'Value ${value} is not allowed. Allowed values are: ${values}',
             mapping=dict(value=value, values=', '.join(allowed)))


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        zope.schema.interfaces.ConstraintNotSatisfied)
@zope.interface.implementer(IErrorMessage)
def country_constraint_not_satisfield(field, exc):
    value = exc.args[0]
    titles = [x.token for x in field.source.factory.getValues()]
    return _(u'Value ${value} is not allowed. Allowed values are: ${values}',
             mapping=dict(value=value, values=', '.join(titles)))


@zope.component.adapter(zope.schema.interfaces.IDate,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(IErrorMessage)
def date_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is no valid date.', mapping=dict(value=value))

@zope.component.adapter(zope.schema.interfaces.IDatetime,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(IErrorMessage)
def datetime_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is no valid datetime. '
             u'Must match to format string "${format}".',
             mapping=dict(value=value, format=DATETIME_FORMAT))

@zope.component.adapter(zope.schema.interfaces.IInt,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(IErrorMessage)
def int_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is not a valid integer number.',
             mapping=dict(value=value))


@zope.component.adapter(zope.schema.interfaces.IDecimal,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(IErrorMessage)
def decimal_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'${value} is not a valid decimal number.',
             mapping=dict(value=value))


@zope.component.adapter(zope.schema.interfaces.IBool,
                        zope.schema.interfaces.WrongType)
@zope.interface.implementer(IErrorMessage)
def bool_wrong_type(field, exc):
    value = exc.args[0]
    return _(u'Value ${value} is not allowed. Allowed values are: ${values}',
             mapping=dict(value=value,
                          values=', '.join(TRUE_VALUES + FALSE_VALUES)))


@zope.component.adapter(None, zope.schema.interfaces.ConstraintNotSatisfied)
@zope.interface.implementer(IErrorMessage)
def email_constraint_not_satisfield(field, exc):
    value = exc.args[0]
    return _(u'${value} is not a valid e-mail address.',
             mapping=dict(value=value))


@zope.component.adapter(None, IndexError)
@zope.interface.implementer(IErrorMessage)
def index_error(field, exc):
    return _(u'Not enough data fields in row.')


def render_error(entity, field_name, exc):
    "Render the error text using the error render adapters."
    obj_title = entity.title

    if field_name is None:
        field = ''
        title = obj_title
        # Need to set the field name to the default here, as a
        # queryMultiAdapter call with name=None behaves strange: it
        # seems to delete all adapters matching object and interface.
        field_name = u''
    else:
        field = entity.getField(field_name)
        title = _('${prefix} -- ${title}',
                  mapping=dict(prefix=obj_title, title=field.title))

    # try named adapter first
    message = zope.component.queryMultiAdapter(
        (field, exc), IErrorMessage, name=field_name)
    if message is None:
        # fallback to default (unnamed) adapter
        message = zope.component.getMultiAdapter(
            (field, exc), IErrorMessage)

    # The rendered errors are stored in a set, so put title and
    # message here, as the message id is equal for all errors.
    mapping = (('title', title), ('message', message))
    return (mapping,
            _(u'${title}: ${message}', mapping=dict(mapping)))


class KeywordBuilder(object):

    KEYWORD_FIELD_NAME = "IcemacAddressbookPersonPerson-0.keywords"
    keyword_index = None
    keywords = None

    def __init__(self, user_data):
        """Expects a mapping between name of the field in the address book and
        field index in import file.

        Example: IcemacAddressbookPersonPerson-0.first_name --> 0
                 IcemacAddressbookAddressHomePageAddress-2.notes --> 11
        """
        if self.KEYWORD_FIELD_NAME in user_data:
            self.keyword_index = user_data[self.KEYWORD_FIELD_NAME]
            self.keywords = zope.component.getUtility(
                icemac.addressbook.interfaces.IKeywords)

    def create(self, data):
        created = []
        if self.keyword_index is None:
            return created
        keyword_data = data[self.keyword_index]
        if keyword_data is None:
            return created
        for keyword_title in split_keywords(keyword_data):
            if self.keywords.get_keyword_by_title(keyword_title, None) is None:
                keyword = icemac.addressbook.utils.create_obj(
                    icemac.addressbook.keyword.Keyword, keyword_title)
                icemac.addressbook.utils.add(self.keywords, keyword)
                created.append(keyword)
        return created


class ImportObjectBuilder(object):
    """Build the objects the user wants to import."""

    def __init__(self, user_data, address_book, entries_number):
        """Expects a mapping between name of the field in the address book and
        field index in import file and
        .

        Example: IcemacAddressbookPersonPerson-0.first_name --> 0
                 IcemacAddressbookAddressHomePageAddress-3.notes --> 11

        Stores the address book field names in dictionaries on
        attributes mapping to the index in the import file.

        address_book ... address book to create the objects in.
        entries_number ... number of entries of each address kind.

        """
        self.address_book = address_book
        self.entries_number = entries_number
        self.import_entities = (
            icemac.ab.importer.browser.wizard.base.get_import_entities())
        for entity in self.import_entities:
            for index in xrange(self.entries_number):
                key = "%s-%s" % (entity.name, index)
                setattr(self, key, {})
        for field_desc, index in user_data.iteritems():
            if index is None:
                continue # field was not selected for import
            prefix, field_name = field_desc.split('.')
            getattr(self, prefix)[field_name] = index

    def create(self, data):
        """Create an object for data.

        data ... import data row, mapping between field index and value."""
        self.errors = set()
        person_entity = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntity,
            name='icemac.addressbook.person.Person')
        person = self._create('%s-0' % person_entity.name, self.address_book,
                              data, True)
        self._validate(person_entity, person)
        for entity in self.import_entities:
            if entity.class_name == 'icemac.addressbook.person.Person':
                # already handled
                continue
            for index in xrange(self.entries_number):
                prefix = "%s-%s" % (entity.name, index)
                main_entry = (index == 0)
                obj = self._create(prefix, person, data, main_entry)
                if main_entry:
                    # set the created address as main address of its kind
                    icemac.addressbook.person.get_default_field(
                        entity.interface).set(person, obj)
                if obj is not None:
                    self._validate(entity, obj)
        # self.errors contains the mapping and a message id, so sort
        # by mapping but return only the message id
        errors = sorted(list(self.errors), key=lambda x: x[0])
        return person, [x[1] for x in errors]

    def _create(self, prefix, parent, data, creation_required):
        field_mapping = getattr(self, prefix)
        len_data = len(data)
        field_values = [data[index]
                        for index in field_mapping.values()
                        if index < len_data and data[index]]
        if not (field_values or creation_required):
            # When there are no values to be stored and creation is
            # not required do nothing.
            return

        # instantiate the object
        entity_name, row_index = prefix.split('-')
        entity = icemac.addressbook.interfaces.IEntity(entity_name)
        obj = icemac.addressbook.utils.create_obj(entity.getClass())

        # set the values
        for field_name, index in field_mapping.iteritems():
            field = entity.getField(field_name)
            try:
                value = data[index]
            except IndexError, e:
                self.errors.add(render_error(entity, None, e))
            else:
                # try named converter first
                conv_value = zope.component.queryMultiAdapter(
                    (value, field), IFieldValue, name=field_name)
                if conv_value is None:
                    # fallback to default (unnamed) converter
                    conv_value = zope.component.getMultiAdapter(
                        (value, field), IFieldValue)
                if conv_value is NONE_REPLACEMENT:
                    conv_value = None
                context = field.interface(obj)
                try:
                    field.set(context, conv_value)
                except zope.interface.Invalid, e:
                    self.errors.add(render_error(entity, field_name, e))

        icemac.addressbook.utils.add(parent, obj)
        return obj

    def _validate(self, entity, obj):
        for field_name, field in entity.getFieldsInOrder():
            context = field.interface(obj)
            field = field.bind(context)
            value = field.get(context)
            try:
                field.validate(value)
            except zope.schema.ValidationError, exc:
                self.errors.add(render_error(entity, field_name, exc))


class FieldsGroup(z3c.form.group.Group):
    "Fields grouped by object."

    def __init__(self, context, request, parent, entity, label, prefix):
        super(FieldsGroup, self).__init__(context, request, parent)
        self.label = label
        self.prefix = prefix
        fields = []
        for field_name, field in entity.getFieldsInOrder():
            choice = zope.schema.Choice(
                title=field.title, description=field.description,
                required=False, source=import_fields)
            choice.__name__ = field_name
            fields.append(choice)
        self.fields = z3c.form.field.Fields(*fields, **dict(prefix=prefix))

    def getContent(self):
        if self.prefix not in self.context:
            self.context[self.prefix] = persistent.mapping.PersistentMapping()
            # We need to store the parent here to find the way back to
            # the parent in `get_reader`.
            self.context[self.prefix]['__parent__'] = self.context
        return self.context[self.prefix]


class MapFields(z3c.form.group.GroupForm,
                icemac.ab.importer.browser.wizard.base.FileSessionStorageStep):
    "Map the fields in the import file to fields in the addressbook."

    label = _(u'Map fields')

    def __init__(self, *args, **kw):
        super(MapFields, self).__init__(*args, **kw)
        session = self.getContent()
        request = self.request
        self.groups = []
        import_entities = (
            icemac.ab.importer.browser.wizard.base.get_import_entities())
        for entity in import_entities:
            if entity.class_name  == 'icemac.addressbook.person.Person':
                entries_number = 1
                main_prefix = ''
            else:
                entries_number = session.get('entries_number', 0)
                main_prefix = _(u'main')
            for index in xrange(entries_number):
                if index == 0:
                    row_title_prefix = main_prefix
                else:
                    row_title_prefix = _(u'other')
                title = _(
                    '${prefix} ${title}',
                    mapping=dict(prefix=row_title_prefix, title=entity.title))
                prefix = '%s-%s' % (entity.name, index)
                self.groups.append(
                    FieldsGroup(session, request, self, entity, title, prefix))

    @property
    def fields(self):
        return z3c.form.field.Fields()

    def update(self):
        super(MapFields, self).update()
        icemac.ab.importer.browser.wizard.base.FileSessionStorageStep.update(
            self)
        if not self.nextURL:
            # no redirect to next step, make sure no imported data exists
            icemac.ab.importer.browser.wizard.base.delete_imported_data(self)

    def applyChanges(self, data):
        super(MapFields, self).applyChanges(data)
        import_object_builder = ImportObjectBuilder(
            data, icemac.addressbook.interfaces.IAddressBook(self.context),
            self.getContent()['entries_number'])
        keyword_builder = KeywordBuilder(data)
        session = self.getContent()
        reader = get_reader(session)
        imported = session['imported'] = []
        imported_keywords = session['imported_keywords'] = []
        import_errors = session['import_errors'] = {}
        session['found_errors'] = False
        for data_row in reader:
            imported_keywords.extend(x.__name__
                                     for x in keyword_builder.create(data_row))
            obj, errors = import_object_builder.create(data_row)
            imported.append(obj.__name__)
            import_errors[obj.__name__] = errors
            if errors:
                session['found_errors'] = True



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

import unittest
import zope.interface.verify
import icemac.ab.importer.interfaces
import icemac.ab.importer.importer


class TestInterfaces(unittest.TestCase):

    def test_importer(self):
        zope.interface.verify.verifyObject(
            icemac.ab.importer.interfaces.IImporter,
            icemac.ab.importer.importer.Importer())



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import icemac.ab.importer.importer
import icemac.ab.importer.interfaces
import icemac.addressbook.interfaces
import zope.component
import zope.container.interfaces


@zope.component.adapter(
    icemac.addressbook.interfaces.IAddressBook,
    zope.container.interfaces.IObjectAddedEvent)
def install_importer(address_book, event=None):
    "Install the importer in the newly created addressbook."
    icemac.addressbook.addressbook.create_and_register(
        address_book, 'importer', icemac.ab.importer.importer.Importer,
        icemac.ab.importer.interfaces.IImporter)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import icemac.ab.importer.interfaces
import zope.component
import zope.interface


class BaseReader(object):
    """Base class for import file readers."""

    zope.interface.implements(icemac.ab.importer.interfaces.IImportFileReader)
    zope.component.adapts(None)

    title = None
    file = None

    def __init__(self, ignored=None):
        """Adapter look-up requires an argument, but we do not need one."""

    @classmethod
    def open(cls, file_handle):
        reader = cls()
        reader.file = file_handle
        return reader

    @classmethod
    def canRead(cls, file_handle):
        try:
            reader = cls.open(file_handle)
            return bool(list(reader.getFieldNames()))
        except:
            return False

    def __del__(self):
        if self.file is not None:
            self.file.close()

    def getFieldNames(self):
        raise NotImplementedError

    def getFieldSamples(self, field_name):
        raise NotImplementedError

    def __iter__(self):
        raise NotImplementedError



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: testing.py 781 2010-01-09 14:24:30Z icemac $

import zope.app.testing.functional


zope.app.testing.functional.defineLayer(
    'ImporterLayer', 'ftesting.zcml', allow_teardown=True)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import icemac.ab.importer.reader.testing
import icemac.ab.importer.reader.csv.csv


class CSVTest(icemac.ab.importer.reader.testing.ReaderTest):

    reader_class = icemac.ab.importer.reader.csv.csv.CSV
    import_file = 'long.csv'
    import_file_short = 'short.csv'



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import icemac.ab.importer.interfaces
import zc.sourcefactory.contextual
import zope.component


class Importers(zc.sourcefactory.contextual.BasicContextualSourceFactory):
    """Source of readers which are able to read the blob."""

    def getValues(self, context):
        # Get all import file readers
        adapters =  zope.component.getAdapters(
            (None, ), icemac.ab.importer.interfaces.IImportFileReader)
        file = icemac.ab.importer.interfaces.IImportFile(context)

        # return only the readers which are able to read the import file
        for name, adapter in adapters:
            if adapter.__class__.canRead(file.openDetached()):
                yield name

    def getTitle(self, context, value):
        adapter = zope.component.getAdapter(
            None, icemac.ab.importer.interfaces.IImportFileReader,
            name=value)

        return adapter.title



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import gocept.cache.property
import icemac.ab.importer.interfaces
import icemac.addressbook.interfaces
import persistent.mapping
import z3c.form.field
import z3c.wizard.wizard
import zope.component
import zope.interface
import zope.security.proxy
import zope.session.interfaces


class ImportWizard(z3c.wizard.wizard.Wizard):

    label = _(u'Import Wizard')

    def setUpSteps(self):
        return [
            z3c.wizard.step.addStep(self, 'editFile', weight=1),
            z3c.wizard.step.addStep(self, 'reader', weight=2),
            z3c.wizard.step.addStep(self, 'map', weight=3),
            z3c.wizard.step.addStep(self, 'review', weight=4),
            z3c.wizard.step.addStep(self, 'complete', weight=5),
            ]


class FileSession(persistent.mapping.PersistentMapping):
    "Session of an import file."

    file = None
    cache = gocept.cache.property.TransactionBoundCache('_cache', dict)


@zope.component.adapter(FileSession)
@zope.interface.implementer(icemac.ab.importer.interfaces.IImportFile)
def file_session_to_import_file(file_session):
    """Get the import file of its session."""
    return file_session.file


def get_file_session(file, request):
    "Get the session associated with the file."
    session = zope.session.interfaces.ISession(request)[
        icemac.addressbook.interfaces.PACKAGE_ID]
    key = 'import_%s' % file.__name__
    file_session = session.get(key, None)
    if file_session is None:
        file_session = FileSession()
        session[key] = file_session
    file_session.file = zope.security.proxy.removeSecurityProxy(file)
    return file_session


class Step(z3c.wizard.step.Step):

    @property
    def fields(self):
        return z3c.form.field.Fields(self.interface)


class FileSessionStorageStep(Step):
    "Step which stores its data in file's session."

    def getContent(self):
        return get_file_session(self.context, self.request)


def get_import_entities():
    "Get the entities relevant for the import."
    name_suffixes = ['person.Person',
                     'address.PostalAddress',
                     'address.PhoneNumber',
                     'address.EMailAddress',
                     'address.HomePageAddress']
    return [zope.component.getUtility(
                icemac.addressbook.interfaces.IEntity,
                name='icemac.addressbook.'+suffix)
            for suffix in name_suffixes]


@zope.interface.implementer(icemac.addressbook.interfaces.IAddressBook)
@zope.component.adapter(icemac.ab.importer.interfaces.IImportFile)
def importfile_to_addressbook(import_file):
    "Adapt import file to address book."
    return import_file.__parent__.__parent__


def delete_imported_data(self):
    """Delete previously imported data."""
    addressbook = icemac.addressbook.interfaces.IAddressBook(self.context)
    session = self.getContent()
    for id in session.get('imported', []):
        try:
            del addressbook[id]
        except KeyError:
            # This can occur when importing the same data again after
            # delting it
            pass
    session['imported'] = []
    keywords = zope.component.getUtility(
        icemac.addressbook.interfaces.IKeywords)
    for id in session.get('imported_keywords', []):
        del keywords[id]
    session['imported_keywords'] = []



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import icemac.ab.importer.browser.wizard.base
import icemac.ab.importer.source
import zope.interface
import zope.schema


class IReaderSettings(zope.interface.Interface):
    "Reader Setttings."

    reader = zope.schema.Choice(
        title=_(u'Import file reader'),
        source=icemac.ab.importer.source.Importers())

    entries_number = zope.schema.Int(
        title=_(u'Number of e.g. phone numbers per person'),
        min=1, max=9, default=1)


class ReaderSettings(
    icemac.ab.importer.browser.wizard.base.FileSessionStorageStep):

    interface = IReaderSettings
    label = _(u'Reader settings')

    @property
    def showBackButton(self):
        """Back button condition."""
        return True

    def doBack(self, action):
        """Process back action and return True on sucess."""
        self.getContent()['edit_file_available'] = True
        return super(ReaderSettings, self).doBack(action)




# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import StringIO
import icemac.ab.importer.interfaces
import icemac.ab.importer.reader.base
import icemac.ab.importer.source
import unittest
import zope.component
import zope.component.globalregistry
import zope.interface
import zope.interface.verify


class DummyReader(icemac.ab.importer.reader.base.BaseReader):

    title = u'Dummy Reader'

    def getFieldNames(self):
        """Get the names of the fields in the file."""
        # canRead requires at least a field name to return True
        return ['field1']

    def getFieldSamples(self, field_name):
        """Get sample values for a field."""
        return []

    def __iter__(self):
        """Iterate over the file."""


class NoXMLReader(DummyReader):

    def getFieldNames(self):
        self.file.seek(0)
        data = self.file.read()
        if data.startswith('<?xml'):
            raise ValueError('No XML!')
        return ['field1']


class DummyImportFile(object):

    zope.interface.implements(icemac.ab.importer.interfaces.IImportFile)

    def __init__(self, data):
        self.data = data

    def openDetached(self):
        return StringIO.StringIO(self.data)


class TestSource(unittest.TestCase):

    def setUp(self):
        self.source = icemac.ab.importer.source.Importers()
        gsm = zope.component.globalregistry.getGlobalSiteManager()
        gsm.registerAdapter(DummyReader, name='dummy')
        gsm.registerAdapter(NoXMLReader, name='no-xml')

    def tearDown(self):
        gsm = zope.component.globalregistry.getGlobalSiteManager()
        gsm.unregisterAdapter(DummyReader, name='dummy')
        gsm.unregisterAdapter(NoXMLReader, name='no-xml')

    def test_dummy_reader(self):
        zope.interface.verify.verifyObject(
            icemac.ab.importer.interfaces.IImportFileReader,
            DummyReader())

    def test_noxml_reader(self):
        zope.interface.verify.verifyObject(
            icemac.ab.importer.interfaces.IImportFileReader,
            NoXMLReader())

    def test_getValue__txt_file(self):
        file = DummyImportFile('text')
        self.assertEqual([u'dummy', u'no-xml'],
                         list(self.source.factory.getValues(file)))

    def test_getValue__xml_file(self):
        file = DummyImportFile('<?xml ...')
        self.assertEqual([u'dummy'],
                         list(self.source.factory.getValues(file)))

    def test_getTitle(self):
        file = DummyImportFile('text')
        self.assertEqual(u'Dummy Reader',
                         self.source.factory.getTitle(file, 'dummy'))
        self.assertEqual(u'Dummy Reader',
                         self.source.factory.getTitle(file, 'no-xml'))



# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: __init__.py 781 2010-01-09 14:24:30Z icemac $
"""Database initialisation and upgrading."""

import zope.app.generations.generations


GENERATION = 0


manager = zope.app.generations.generations.SchemaManager(
    minimum_generation=GENERATION,
    generation=GENERATION,
    package_name='icemac.ab.importer.generations')



# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

import icemac.ab.importer.install
import icemac.addressbook.addressbook
import icemac.addressbook.testing
import zope.component.globalregistry
import zope.container.contained
import zope.traversing.adapters


class TestInstall(icemac.addressbook.testing.FunctionalTestCase):

    def assertLocalUtility(self, ab, iface):
        self.assertTrue(
            icemac.addressbook.addressbook.utility_locally_registered(ab, iface)
            )

    def assertAttribute(self, ab, attribute, iface):
        self.assertTrue(iface.providedBy(getattr(ab, attribute)))
        self.assertLocalUtility(ab, iface)

    def check_addressbook(self, ab):
        self.assertAttribute(
            ab, 'importer', icemac.ab.importer.interfaces.IImporter)

    def setUp(self):
        super(TestInstall, self).setUp()
        root = self.getRootFolder()
        root['ab'] = self.ab = icemac.addressbook.utils.create_obj(
            icemac.addressbook.addressbook.AddressBook)

    def test_create(self):
        icemac.addressbook.addressbook.create_address_book_infrastructure(
            self.ab)
        icemac.ab.importer.install.install_importer(self.ab)
        self.check_addressbook(self.ab)

    def test_recall_create(self):
        icemac.addressbook.addressbook.create_address_book_infrastructure(
            self.ab)
        icemac.ab.importer.install.install_importer(self.ab)
        icemac.ab.importer.install.install_importer(self.ab)
        self.check_addressbook(self.ab)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import unittest
import icemac.ab.importer.browser.table
import datetime
import pytz
import zope.publisher.browser


class DummyDC(object):

    modified = None


class TestModifiedColumnLocalTime(unittest.TestCase):

    def setUp(self):
        request = zope.publisher.browser.TestRequest()
        self.column = icemac.ab.importer.browser.table.ModifiedColumnLocalTime(
            None, request, None)
        self.item = DummyDC()
        self.item.modified = datetime.datetime(
            2003, 5, 3, 11, 12, 13, tzinfo=pytz.timezone('UTC'))

    def tearDown(self):
        # timezones seems to store information globally
        zope.publisher.browser.TestRequest().locale.dates.timezones.pop(
            u'Europe/Berlin', None)

    def test_no_item(self):
        self.assertEqual(u'', self.column.getValue(None))

    def test_no_value(self):
        self.item.modified = None
        self.assertEqual(u'', self.column.getValue(self.item.modified))

    def test_utc_timezone(self):
        value = self.column.getValue(self.item)
        self.assertEqual(self.item.modified, value)
        self.assertEqual('UTC', value.tzinfo.zone)

    def test_local_timezone(self):
        self.column.request.locale.dates.timezones[u'Europe/Berlin'] = (
            zope.i18n.locales.LocaleTimeZone(u'Europe/Berlin'))
        value = self.column.getValue(self.item)
        self.assertEqual(self.item.modified, value)
        self.assertEqual('Europe/Berlin', value.tzinfo.zone)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import icemac.ab.importer.browser.wizard.base
import icemac.addressbook.browser.file.file
import icemac.addressbook.file.interfaces


class EditFile(icemac.ab.importer.browser.wizard.base.Step):

    interface = icemac.addressbook.file.interfaces.IFile
    label = _(u'Edit import file')

    @property
    def available(self):
        return icemac.ab.importer.browser.wizard.base.get_file_session(
            self.context, self.request).get('edit_file_available', False)

    def applyChanges(self, data):
        super(EditFile, self).applyChanges(data)
        icemac.addressbook.browser.file.file.update_blob(
            self.widgets['data'], self.context)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: test_doctests.py 781 2010-01-09 14:24:30Z icemac $

import icemac.addressbook.testing
import icemac.ab.importer.browser.testing


def test_suite():
    return icemac.addressbook.testing.FunctionalDocFileSuite(
        "importer.txt",
        "masterdata.txt",
        package='icemac.ab.importer.browser',
        layer=icemac.ab.importer.browser.testing.ImporterLayer,
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import zope.component
import icemac.ab.importer.interfaces


class Readers(object):

    def readers(self):
        readers =  zope.component.getAdapters(
            (None, ), icemac.ab.importer.interfaces.IImportFileReader)
        for name, reader in readers:
            yield reader.title



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import zope.traversing.api
import zope.traversing.browser.absoluteurl
import z3c.wizard.step


class Complete(z3c.wizard.step.Step):

    label = _(u'Complete')

    def applyChanges(self, data):
        super(Complete, self).applyChanges(data)
        self.wizard.nextURL = zope.traversing.browser.absoluteurl.absoluteURL(
            zope.traversing.api.getParent(self.context), self.request)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: install.py 781 2010-01-09 14:24:30Z icemac $
"""Initial generation."""

__docformat__ = "reStructuredText"

import icemac.ab.importer.install
import icemac.addressbook.interfaces
import zope.app.generations.utility


def evolve(context):
    """Installs the importer into each existing address book."""
    root = zope.app.generations.utility.getRootFolder(context)
    address_books = zope.app.generations.utility.findObjectsProviding(
        root, icemac.addressbook.interfaces.IAddressBook)
    for address_book in address_books:
        icemac.ab.importer.install.install_importer(address_book, None)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import icemac.addressbook.browser.table
import z3c.table.column
import icemac.ab.importer.browser.table


class Overview(icemac.addressbook.browser.table.PageletTable):

    no_rows_message = _(u'No import files uploaded, yet.')

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TitleLinkColumn,
                'file', weight=1),
            z3c.table.column.addColumn(
                self, z3c.table.column.GetAttrColumn, 'mimeType', weight=2,
                header=_(u'MIME type'), attrName='mimeType'),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TruncatedContentColumn,
                'notes', weight=3, header=_(u'Notes'), attrName='notes',
                length=50),
            z3c.table.column.addColumn(
                self, icemac.ab.importer.browser.table.ModifiedColumnLocalTime,
                'modified', weight=5),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.DeleteLinkColumn,
                'delete', weigth=6),
            z3c.table.column.addColumn(
                self, z3c.table.column.LinkColumn, 'import', weight=200,
                header=_(u''), linkContent=_(u'Import'),
                linkName='@@import'),
            ]

    @property
    def values(self):
        return self.context.values()



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import zope.container.btree
import zope.interface
import icemac.ab.importer.interfaces


class Importer(zope.container.btree.BTreeContainer):
    "Importer containing files for import."
    zope.interface.implements(icemac.ab.importer.interfaces.IImporter)

    file_marker_interface = icemac.ab.importer.interfaces.IImportFile



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

import os.path
import setuptools

def read(*path_elements):
    return "\n\n" + file(os.path.join(*path_elements)).read()

version = '1.3.0'
long_description = (
    read('README.txt') +
    read('CHANGES.txt')
    )

setuptools.setup(
    name='icemac.ab.importer',
    version=version,
    description="Import infrastructure for icemac.addressbook",
    long_description=long_description,
    keywords='icemac.addressbook',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    url='http://pypi.python.org/pypi/icemac.ab.importer',
    license='ZPL 2.1',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Paste',
        'Framework :: Zope3',
        'License :: OSI Approved',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.5',
        ],
    packages=setuptools.find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages = ['icemac', 'icemac.ab'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'gocept.cache',
        'icemac.addressbook > 1.2.0',
        'icemac.truncatetext >= 0.2',
        'icemac.ab.locales >= 0.2',
        'pytz',
        'setuptools',
        'z3c.wizard',
        'zc.sourcefactory',
        'zope.container',
        'zope.interface',
        'zope.schema',
        'zope.app.generations',
        ],
    extras_require = dict(
        test=[
            'icemac.addressbook [test]',
            'zope.app.testing',
            'zope.testing >= 3.8.0',
            'zope.traversing',
            ]),
    )



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from __future__ import absolute_import
from icemac.addressbook.i18n import MessageFactory as _
import csv
import datetime
import icemac.ab.importer.reader.base
import time


def as_unicode(method):
    def decorated(*args):
        result = method(*args)
        encoding = args[0].encoding
        for value in result:
            yield unicode(value, encoding)
    return decorated


def as_data(method):
    def decorated(*args):
        def convert(value):
            if value == '':
                return None
            try:
                time_tuple = time.strptime(value.strip(), date_format)
            except:
                # no date value, do normal convert
                return unicode(value, encoding)
            else:
                return datetime.date(*time_tuple[:3])

        result = method(*args)
        encoding = args[0].encoding
        date_format = args[0].date_format
        for value in result:
            yield [convert(x) for x in value]

    return decorated


class CSV(icemac.ab.importer.reader.base.BaseReader):
    "Read CSV import files."

    title = _(u'CSV file (comma separated fields, ISO-dates, UTF-8 encoded)')
    encoding = 'utf-8'
    date_format = '%Y-%m-%d'

    def _getReader(self):
        self.file.seek(0)
        return csv.reader(self.file)

    @as_unicode
    def getFieldNames(self):
        # get only first line, as it contains the field names
        return self._getReader().next()

    @as_unicode
    def getFieldSamples(self, field_name):
        index = list(self.getFieldNames()).index(field_name)
        reader = self._getReader()
        # skip first line (field names)
        reader.next()
        num = 1
        for fields in reader:
            yield fields[index]
            if num >= 3:
                return
            num += 1

    @as_data
    def __iter__(self):
        reader = self._getReader()
        # skip first line (field names)
        reader.next()
        for fields in reader:
            yield fields
